/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef _GCTL_GRAV_KERNEL_POLYGON_H
#define _GCTL_GRAV_KERNEL_POLYGON_H

#include "gm_data.h"

namespace gctl
{
    /**
     * @brief      计算直角坐标下的二维多边形单元体重力观测值正演
     * 
     * @param out_obs             输出的重力观测值
     * @param cor_vert            多边形的角点顶点数组（逆时针排序）
     * @param obsp                二维直角坐标下的观测点数组
     * @param rho                 多边形的密度
     * @param comp_id             待计算的重力场分量 默认为重力值
     */
	void gobser(array<double> &out_obs, const array<vertex2dc> &cor_vert, const array<point2dc> &obsp, 
		const double &rho, gravitational_field_type_e comp_id = Vz);
}

#endif // _GCTL_GRAV_KERNEL_POLYGON_H