/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef _GCTL_GRAV_KERNEL_TESSEROID_H
#define _GCTL_GRAV_KERNEL_TESSEROID_H

#include "gm_data.h"

#ifdef GCTL_POTENTIAL_TESS

namespace gctl
{
	/**
	 * @brief      计算球坐标下的Tesseroid单元体正演核矩阵
	 * 
	 * @note       Tesseroid类型仅用于球坐标下的重力数据正演
	 *
	 * @param      out_kernel     输出的核矩阵
	 * @param[in]  ele            Tesseroid单元体数组
	 * @param[in]  ops            球坐标下的观测点数组
	 * @param[in]  comp_id        待计算的重力场分量 默认为重力值
	 * @param[in]  show_progress  是否在终端显示进度条 默认为真
	 */
	void gkernel(matrix<double> &out_kernel, const array<tesseroid> &ele, const array<point3ds> &obsp, 
		gravitational_field_type_e comp_id = Vz, verbose_type_e verbose = FullMsg);
    
    /**
	 * @brief      计算球坐标下的Tesseroid单元体正演重力数据
	 * 
	 * @note       Tesseroid类型仅用于球坐标下的重力数据正演
	 *
	 * @param      out_kernel     输出的重力数据
	 * @param[in]  ele            Tesseroid单元体数组
	 * @param[in]  obsp           球坐标下的观测点数组
	 * @param[in]  comp_id        待计算的重力场分量 默认为重力值
	 * @param[in]  verbose        返回信息层级
	 */
	void gobser(array<double> &out_obs, const array<tesseroid> &ele, const array<point3ds> &obsp, 
		const array<double> &rho, gravitational_field_type_e comp_id = Vz, verbose_type_e verbose = FullMsg);
}

#endif // GCTL_POTENTIAL_TESS

#endif // _GCTL_GRAV_KERNEL_TESSEROID_H