﻿/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2022  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#ifndef _GM_REGULAR_MESH_SPH_3D_H
#define _GM_REGULAR_MESH_SPH_3D_H

#include "gctl/mesh/regular_mesh_sph_3d.h"

#include "gkernel_tesseroid.h"
#include "mkernel_tesseroid.h"

namespace gctl
{
	class gm_regular_mesh_sph_3d : public regular_mesh_sph_3d
	{
	protected:
		array<mag_tesseroid> mag_ele_;
	
	public:
		gm_regular_mesh_sph_3d();
		gm_regular_mesh_sph_3d(std::string in_name, std::string in_info, double lon_min, 
            double lat_min, double rad_min, double lon_size, double lat_size, double rad_size, 
            int lon_bnum, int lat_bnum, int rad_bnum);
		virtual ~gm_regular_mesh_sph_3d();

#ifdef GCTL_POTENTIAL_TESS
		void gkernel(matrix<double> &out_kernel, const array<point3ds> &obs, 
			gravitational_field_type_e comp_id = Vz, verbose_type_e verbose = FullMsg);
		void gobser(array<double> &out_data, std::string data_name, const array<point3ds> &obs, 
			gravitational_field_type_e comp_id = Vz, verbose_type_e verbose = FullMsg);
#endif // GCTL_POTENTIAL_TESS

#ifdef GCTL_POTENTIAL_MAGTESS
        void magkernel(matrix<double> &out_kernel, const array<point3ds> &obs, 
            const array<double> &Bx, const array<double> &By, const array<double> &Bz, 
            magnetic_field_type_e comp_id = Za, verbose_type_e verbose = FullMsg);
        void magobser(array<double> &out_data, std::string data_name, const array<point3ds> &obs, 
            const array<double> &Bx, const array<double> &By, const array<double> &Bz, 
            magnetic_field_type_e comp_id = Za, verbose_type_e verbose = FullMsg);
#endif // GCTL_POTENTIAL_MAGTESS
	};
}

#endif //_GM_REGULAR_MESH_SPH_3D_H